package cz.zcu.kiv.eegdsp.fourier;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.math.complex.Complex;

import cz.zcu.kiv.eegdsp.common.ISignalProcessingResult;

/**
 * Result of Fast Fourier Transformation.
 * 
 * @author Petr Kellnhofer, A10N0018P
 * @date 2.1.2011
 *
 */
public class FastFourierResult implements ISignalProcessingResult {
	
	/**
	 * Complex coefficients from FFT.
	 */
	private Complex[] coefficients;

	/**
	 * Gets complex coefficients from FFT.
	 * @return complex coefficients from FFT
	 */
	public Complex[] getCoefficients() {
		return coefficients;
	}
	
	/**
	 * Creates new result instance.
	 * 
	 * @param coefficients complex coefficients from FFT
	 */
	public FastFourierResult(Complex[] coefficients) {
		if (coefficients == null) {
			throw new IllegalArgumentException("Coefficients must be provided.");
		}
		
		// clear nulls
		for (int i = 0; i < coefficients.length; i++) {
			if (coefficients[i] == null) {
				coefficients[i] = new Complex(0.0, 0.0);
			}
		}
		
		this.coefficients = coefficients;
	}

	/**
	 * Extracts data to array of two dimensional arrays
	 * 
	 * 0 => real parts of coefficients
	 * 1 => imag parts of coefficients
	 * 
	 * @return array of two dimensional arrays
	 */
	@Override
	public double[][][] toArray() {
		double[][][] res = new double[2][1][this.coefficients.length];
		
		for (int i = 0; i < this.coefficients.length; i++) {
			res[0][0][i] = this.coefficients[i].getReal();
			res[1][0][i] = this.coefficients[i].getImaginary();
		}
		
		return res;
	}

	/**
	 * Extracts data to map of two dimensional arrays.
	 * 
	 * @return map of two dimensional arrays
	 */
	@Override
	public Map<String, Double[][]> toHashMap() {
		HashMap<String, Double[][]> map = new HashMap<String, Double[][]>();
		
		Double[][] array = new Double[this.coefficients.length][2];
		for (int i = 0; i < this.coefficients.length; i++) {
			array[i][0] = Double.valueOf(this.coefficients[i].getReal());
			array[i][1] = Double.valueOf(this.coefficients[i].getImaginary());
		}

		map.put("coefficients", array);
		
		return map;
	}

	/**
	 * Prints data do string.
	 * 
	 * @return string
	 */
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		
		builder.append("FastFourierTransformation:\n");
		
		builder.append("\tCoefficients:\n");
		builder.append("[");
		for (int i = 0; i < this.coefficients.length; i++) {
			
				Complex c = this.coefficients[i];
			
				if (c.getImaginary() >= 0)
				{
					builder.append(c.getReal() + " + " + Math.abs(c.getImaginary()) + "i");
				}
				else
				{
					builder.append(c.getReal() + " - " + Math.abs(c.getImaginary()) + "i");
				}
			
			if (i != this.coefficients.length - 1) {
				builder.append(", ");
			}
		}
		
		return builder.toString();
	}
}
