package cz.zcu.kiv.eegdsp.matchingpursuit;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import cz.zcu.kiv.eegdsp.common.ISignalProcessor;
import cz.zcu.kiv.eegdsp.matchingpursuit.algorithm.Base;
import cz.zcu.kiv.eegdsp.matchingpursuit.algorithm.DecompositionCollection;
import cz.zcu.kiv.eegdsp.matchingpursuit.algorithm.GaborBase;
import cz.zcu.kiv.eegdsp.matchingpursuit.algorithm.MPFFT;
import cz.zcu.kiv.eegdsp.utils.Mathematic;

/**
 * Interface for Matching Pursuit alghorithm.
 * 
 * @author Petr Kellnhofer, A10N0018P
 * @date 2.1.2011
 *
 */
public class MatchingPursuit implements ISignalProcessor {
	
	/**
	 * Private instance of singleton.
	 */
	private static MatchingPursuit instance;
	
	/**
	 * Collection of bases.
	 */
	private List<Base> bases;
	
	/**
	 * Gets editable base collection.
	 * Bases are used in matching pursuit alghorithm
	 * as sources of decomposition atoms.
	 * 
	 * Default collection contains Gabor Base
	 * for Gabor atoms.
	 * 
	 * @return base collection
	 */
	public List<Base> getBases() {
		return bases;
	}

	/**
	 * Number of iterations.
	 * 
	 * Each iteration adds one atom.
	 */
	private int iterationCount;
	
	/**
	 * Gets number of iterations.
	 * 
	 * Each iteration adds one atom.
	 * 
	 * @return number of iterations
	 */
	public int getIterationCount() {
		return iterationCount;
	}

	/**
	 * Sets number of iterations.
	 * 
	 * Each iteration adds one atom.
	 * 
	 * @param iterationCount number of iterations
	 */
	public void setIterationCount(int iterationCount) {
		if (iterationCount < 1) {
			throw new IllegalArgumentException("Number of iterations must be >= 1");
		}
		
		this.iterationCount = iterationCount;
	}

	/**
	 * Private constructor.
	 * Sets base to be Gabor base.
	 */
	private MatchingPursuit() {
		this.iterationCount = 10;
		
		this.bases = new ArrayList<Base>();
		this.bases.add(new GaborBase()); // default base
	}
	
	/**
	 * Processes signal using matching pursuit alghorithm.
	 * Returns result with atom collection.
	 * Handles signal extension to power of 2 if neccessary.
	 * 
	 * @return result structure
	 */
	@Override
	public MatchingPursuitResult processSignal(double[] signal) {
		if (signal == null) {
			throw new IllegalArgumentException("Signal must be provided.");
		}
		
		// zero padding to length match power of 2
		int power2 = Mathematic.newMajorNumberOfPowerBase2(signal.length);
		if (power2 != signal.length) {
			signal = Arrays.copyOf(signal, power2);
		}
		
		MPFFT matchingPursuit = new MPFFT((Base[])bases.toArray(new Base[0]));
		
		DecompositionCollection dc = matchingPursuit.doMP(signal, this.iterationCount);
		
		return new MatchingPursuitResult(dc);
	}
	
	/**
	 * Gets instance of this singleton.
	 * 
	 * @return instance of this singleton
	 */
	public static MatchingPursuit getInstance() {
		if (instance == null) {
			instance = new MatchingPursuit();
		}
		
		return instance;
	}
}
