package cz.zcu.kiv.eegdsp.matchingpursuit;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import cz.zcu.kiv.eegdsp.common.ISignalProcessingResult;
import cz.zcu.kiv.eegdsp.matchingpursuit.algorithm.Atom;
import cz.zcu.kiv.eegdsp.matchingpursuit.algorithm.DecompositionCollection;

/**
 * Result of matching pursuit decomposition.
 * 
 * @author Petr Kellnhofer, A10N0018P
 * @date 2.1.2011
 *
 */
public class MatchingPursuitResult implements ISignalProcessingResult {
	
	/**
	 * Inner decomposition collection of atoms.
	 */
	private DecompositionCollection decompositionCollection;
	
	/**
	 * Gets inner decomposition collection of atoms.
	 * @return inner decomposition collection of atoms
	 */
	public DecompositionCollection getDecompositionCollection() {
		return decompositionCollection;
	}

	/**
	 * Creates result object from decomposition collection created during mathcing pursuit algorithm. 
	 * 
	 * @param dcCollection decomposition collection from MP.
	 */
	public MatchingPursuitResult(DecompositionCollection dcCollection) {
		if (dcCollection == null) {
			throw new IllegalArgumentException("Decomposition collection must provided.");
		}
		
		this.decompositionCollection = dcCollection;
	}
	
	/**
	 * Gets atoms used in decomposition.
	 * 
	 * @return list of atoms used
	 */
	public List<Atom> getAtoms() {
		List<Atom> atoms = new ArrayList<Atom>(this.decompositionCollection.size());
		
		for (Atom atom : this.decompositionCollection) {
			atoms.add(atom);
		}
		
		return atoms;
	}
	
	/**
	 * Gets reconstruction of input signal using all atoms.
	 * 
	 * @return reconstruction of input signal
	 */
	public double[] getReconstruction() {
		return this.decompositionCollection.getReconstruction(this.decompositionCollection.size());
	}

	/**
	 * Exports data from result to array.
	 * 
	 * 0 => atom values
	 * 1 => reconstructed signal
	 * 
	 * @return array export of data
	 */
	@Override
	public double[][][] toArray() {
		double[][][] res = new double[5][][];
		
		res[0] = new double[this.decompositionCollection.size()][];
		
		for (int i = 0; i < this.decompositionCollection.size(); i++) {
			res[0][i] = this.decompositionCollection.getAtomHistory(i);
		}
		
		res[1] = new double[1][];
		res[1][0] = this.getReconstruction();
		
		return res;
 	}

	/**
	 * Exports values to string keyed map.
	 * 
	 * @return map of values
	 */
	@Override
	public Map<String, Double[][]> toHashMap() {
		Map<String, Double[][]> res = new HashMap<String, Double[][]>();
		
		Double[][] atoms = new Double[this.decompositionCollection.size()][this.decompositionCollection.getInputSignal().length];
		for (int i = 0; i < this.decompositionCollection.size(); i++) {
			double[] atom = this.decompositionCollection.getAtomHistory(i);
			
			for (int j = 0; j < atom.length && j < atoms[i].length; j++) {
				atoms[i][j] = Double.valueOf(atom[j]);
			}
		}
		res.put("atoms", atoms);
		
		double[] reconstruction = this.getReconstruction();
		Double[][] out = new Double[1][reconstruction.length];
		
		for (int i = 0; i < reconstruction.length; i++) {
			out[0][i] = reconstruction[i];
		}
		res.put("reconstruction", out);
		
		return res;
	}

	/**
	 * Gets string representation of results.
	 */
	@Override
	public String toString() {
		StringBuilder res = new StringBuilder();
		
		res.append("Matching pursuit:\n");
		res.append("\tAtoms:\n");
		
		int i = 0;
		for (Atom atom : this.decompositionCollection) {
			res.append(String.format("\t%d: %s\n", i, atom.toString()));
			i++;
		}
		
		res.append("\tReconstructed signal:\n");
		double[] reconstruction = this.getReconstruction();
		res.append(Arrays.toString(reconstruction));
		
		return res.toString();
	}
}
