package cz.zcu.kiv.eegdsp.matchingpursuit.algorithm;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
/**
 * Collection of atoms creates during decomposition of input signal.
 * 
 * @author Tom ondk
 */
public class DecompositionCollection implements Iterable<Atom>
{
	/**
	 * List of atoms found in signal.
	 */
	private List<Atom> atoms;
	
	/**
	 * Original input signal.
	 */
	private double[] signal;
	
	/**
	 * Gets original input signal.
	 * @return input signal
	 */
	public double[] getInputSignal() {
		return this.signal;
	}
	
	/**
	 * Empty constructor for new signal processing.
	 * 
	 * @param signal input signal
	 */
	public DecompositionCollection(double[] signal)
	{
		if (signal == null) {
			throw new IllegalArgumentException("Signal must be provided.");
		}
		
		atoms = new ArrayList<Atom>();
		this.signal = signal;
	}
	/**
	 * Gets input signal for matching pursuit iteration specified by the <code>index</code> parameter.
	 * 
	 * @param index iteration number of matching pursuit algorithm
	 * @return signal for index-th iteration of MP
	 */
	public double[] getSignalHistory(int index)
	{
		// copy input signal
		double[] retval = Arrays.copyOf(signal, signal.length);
		double[] current = null;
		
		// to index or number of atoms
		for (int i = 0; i < index && index <= atoms.size(); i++)
		{
			// gets values of atom
			current = atoms.get(i).getValues(retval.length);
			
			// subtract atom
			for (int j = 0; j < retval.length; j++)
			{
				retval[j] -= current[j];
			}
		}
		
		return retval;
	}
	/**
	 * Gets input signal reconstruction back from specified number of atoms.
	 * 
	 * @param level number of atoms to use
	 * @return reconstructed signal
	 */
	public double[] getReconstruction(int level)
	{
		double[] retval = new double[signal.length];
		Arrays.fill(retval, 0);
		double[] current;
		
		// through atoms
		for (int j = 0; j < level && j < atoms.size(); j++)
		{
			// get and add atom
			current = atoms.get(j).getValues(retval.length);
			for (int i = 0; i < retval.length; i++)
			{
				retval[i] += current[i];
			}
		}
		
		return retval;
	}
	/**
	 * Gets difference between the original signal and reconstruction using <code>level</code> atoms.
	 * @param level number of atoms to use
	 * @return difference vertex between signals
	 */
	public double[] getReconstructionDifference(int level)
	{
		// reconstruct
		double[] reconstruction = getReconstruction(level);
		double[] difference = new double[reconstruction.length];
		Arrays.fill(difference, 0);
		
		// diff
		for (int i = 0; i < difference.length; i++)
		{
			difference[i] = signal[i] - reconstruction[i];
		}
		
		return difference;
	}
	/**
	 * Gets values of atom on specified <code>index</code>.
	 * @param index index of atoms used
	 * @return function values of atom
	 */
	public double[] getAtomHistory(int index)
	{
		return atoms.get(index).getValues(signal.length);
	}
	/**
	 * Gets number of atoms used in decomposition.
	 * @return number of atoms
	 */
	public int size()
	{
		return atoms.size();
	}
	/**
	 * Adds atom to decomposition
	 * @param atom reference to added atom
	 */
	public void addAtom(Atom atom)
	{
		atoms.add(atom);
	}
	
	/**
	 * Gets atom used in decomposition on specified <code>index</code>. 
	 * @param index index of atom
	 * @return atom on index
	 */
	public Atom getAtom(int index)
	{
		return atoms.get(index);
	}
	
	/**
	 * Creates string represenation of class.
	 * @return Gets text representation of atoms.
	 */
	@Override
	public String toString()
	{
		String retval = "";
		
		for (Atom a: atoms)
		{
			retval += a.toString() + "\n";
		}
		
		return retval;
	}
	
	/**
	 * Gets iterator to atom list.
	 * @return iterator of atom list
	 */
	@Override
	public Iterator<Atom> iterator()
	{
		return atoms.iterator();
	}
}
