package cz.zcu.kiv.eegdsp.matchingpursuit.algorithm;

import org.apache.commons.math.MathException;
import org.apache.commons.math.complex.Complex;
import org.apache.commons.math.transform.FastFourierTransformer;

import cz.zcu.kiv.eegdsp.utils.Mathematic;

/**
 * Base of Gabor functions.
 * 
 * @author Tom ondk
 */
public class GaborBase implements Base {

	/**
	 * Ignored difference from zero. Values (<code>0 +- ZERO_ALLOWANCE</code>) considered equal to zero.
	 */
	public static final double ZERO_ALLOWANCE = 1.0e-7F;

	/**
	 * Gets Gabor atom best fitting signal. Uses alghorithm published in:
	 * 
	 * FERRANDO, S. E.  KOLASA, L. A.  KOVAEVI, N. 
	 * Algorithm 820: A Flexible Implementation of Matching Pursuit for Gabor Functions on the Interval. 
	 * ACM Transactions on Mathematical Software, 
	 * September 2002, vol. 28, no. 3, 
	 * s. 337  353.
	 * 
	 * @param signal input signal
	 * @return best fitting Gabor signal
	 */
	@Override
	public Atom getOptimalAtom(double[] signal)
	{
		GaborsAtom gaborAtom = null;
		double scale, position, positionDiferencial = 0;
		int n = signal.length;
		double a1 = 0, a = 0, b = 0, b1 = 0, frequency = 0, phase = 0, coef = 0, product = 0;
		double[] x = new double[n];
		double[] y = new double[n];
		Complex[] X = new Complex[n];
		Complex[] Y = new Complex[n];
		double FP = 0, FQ = 0, PP = 0, QQ = 0, PQ = 0;
		scale = n;
		FastFourierTransformer fft = new FastFourierTransformer();
		
		while (scale > 1)
		{
			positionDiferencial = scale / 2;
			for (position = 0; position <= n; position += positionDiferencial)
			{
				for (int j = 0; j < n; j++)
				{
					double g = Mathematic.valueOfGaussianWindow((j - position) / scale);
					x[j] = signal[j] * g;
					y[j] = Math.pow(g, 2);
				}
				
				try {
					X = fft.transform(x);
					Y = fft.transform(y);
				} catch (IllegalArgumentException e) {
					throw new RuntimeException(e.getMessage());
				} catch (MathException e) {
					throw new RuntimeException(e.getMessage());
				}
								
				for (int k = 0; k < n / 2; k++)
				{
					FP = X[k].getReal();
					FQ = -X[k].getImaginary();
					double c = Y[0].getReal();
					
					if (k <= n / 2 - 1)
					{
						PP = (c + Y[2 * k].getReal()) / 2;
						QQ = (c - Y[2 * k].getReal()) / 2;
						PQ = -Y[2 * k].getImaginary() / 2;
					}
					else
					{
						PP = (c + Y[2 * k - n].getReal()) / 2;
						QQ = (c - Y[2 * k - n].getReal()) / 2;
						PQ = -Y[2 * k - n].getImaginary() / 2;
					}
					
					frequency = 2 * Math.PI * k / n;
					a = FP;
					b = FQ;
					a1 = a * QQ - b * PQ;
					b1 = b * PP - a * PQ;
					
					if (frequency == 0)
					{
						phase = 0;
						product = a / Math.sqrt(PP);
					}
					else if (a1 < ZERO_ALLOWANCE)
					{
						phase = Math.PI / 2;
						product = -b / Math.sqrt(QQ);
					}
					else
					{
						phase = Math.atan(-b1 / a1);
						product = (a * a1 + b * b1)
								/ Math.sqrt(Math.pow(a1, 2) * PP 
								+ Math.pow(b1, 2) * QQ + 2 * a1 * b1 * PQ);
					}
					
					if (product >= coef && product != Double.POSITIVE_INFINITY)
					{
						coef = product;
						gaborAtom = new GaborsAtom(scale, position, frequency, phase);
						gaborAtom.setModulus(coef);
					}
				}
			}
			scale = scale / 2;
		}
		
		double difference = 0;
		double[] valuesOfAtom = gaborAtom.getValues(signal.length);
		
		for (int oi = 0; oi < signal.length; oi++)
		{
			difference += Math.abs(signal[oi] - valuesOfAtom[oi]);
		}
		
		gaborAtom.setDifference(difference);
		
		return gaborAtom;
	}
}
