package cz.zcu.kiv.eegdsp.matchingpursuit.algorithm;

import cz.zcu.kiv.eegdsp.utils.Mathematic;

/**
 * Gabor atom representation.
 * 
 * @author Tom ondk
 */
public class GaborsAtom extends Atom
{
	/*
	 * Attributes 
	 */
	
	/**
	 * Frequency.
	 */
	private double frequency;

	/**
	 * Gets frequency.
	 * 
	 * @return frequency
	 */
	public double getFrequency()
	{
		return frequency;
	}
	
	/**
	 * Sets frequency.
	 * 
	 * @param frequency frequency
	 */
	public void setFrequency(double frequency)
	{
		this.frequency = frequency;
	}
	
	/**
	 * Phase shift.
	 */
	private double phase;
	
	/**
	 * Gets phase shift.
	 * 
	 * @return phase shift
	 */
	public double getPhase()
	{
		return phase;
	}
	
	/**
	 * Sets phase shift.
	 * 
	 * @param phase phase shift
	 */
	public void setPhase(double phase)
	{
		this.phase = phase;
	}
	
	/**
	 * Modulus
	 */
	private double modulus;
	
	/**
	 * Gets modulus
	 * 
	 * @return modulus
	 */
	public double getModulus()
	{
		return modulus;
	}
	
	/**
	 * Sets modulus.
	 * 
	 * @param modulus modulus
	 */
	public void setModulus(double modulus)
	{
		this.modulus = modulus;
	}
	
	/*
	 * Constructor
	 */

	/**
	 * Creates Gabor atom.
	 * 
	 * @param scale scale
	 * @param position position
	 * @param frequency frueqency
	 * @param phase phase shift
	 */
	public GaborsAtom(double scale, double position, double frequency, double phase)
	{
		this.scale = scale;
		this.position = position;
		this.frequency = frequency;
		this.phase = phase;
	}
	
	/**
	 * Empty constructor,
	 */
	public GaborsAtom(){};
	
	/*
	 * Methods.
	 */
	
	/**
	 * Gets values in all times from 0 to length.
	 * 
	 * @param length length of evaluation
	 */
	@Override
	public double[] getValues(int length)
	{
		double sum = 0;
		
		// evaluate atom
		for (int j = 0; j < length; j++)
		{
			sum += Math.pow(this.evaluate(j), 2);
		}
		
		sum = this.getModulus() / Math.sqrt(sum);
		
		double dec;
		double[] decArray = new double[length];
		
		// output
		for (int j = 0; j < length; j++)
		{
			dec = this.evaluate(j) * sum;
			decArray[j] = dec;
		}
		return decArray;
	}
	
	/**
	 * Subtracts this atom from signal.
	 * 
	 * @param signal input signal (input/output parameter)
	 */
	@Override
	public void subtractFrom(double[] signal)
	{
		double sum = 0;
		
		// evaluate this atom in signal positions.
		for (int j = 0; j < signal.length; j++)
		{
			sum += Math.pow(this.evaluate(j), 2);
		}
		
		sum = this.getModulus() / Math.sqrt(sum);
		
		double dec;
		
		// subtract this atom from signal
		for (int j = 0; j < signal.length; j++)
		{
			dec = this.evaluate(j) * sum;
			signal[j] -= dec;
		}
	}
	
	
	/**
	 * Evaluates Gauss window in time depending on values of <code>scale</code>, 
	 * <code>position</code>, <code>frequency</code> a <code>phase</code> attributes.
	 * 
	 * @param numberOfSample time to evaluate
	 * @return value of this atom
	 */
	@Override
	public double evaluate(double numberOfSample)
	{
		return Mathematic.valueOfGaussianWindow((numberOfSample - position) / scale)
				* Math.cos(numberOfSample * frequency + phase);
	}
	
	/**
	 * Gets text representation of atom.
	 * @return string representation
	 */
	@Override
	public String toString()
	{
		return "Gabors atom - position: " + position 
		+ ", scale: " + scale 
		+ ",  frequency: "  + frequency
		+ ", modulus: " + modulus
		+ ", phase: "
		+ phase;
	}
}
