package cz.zcu.kiv.eegdsp.matchingpursuit.algorithm;

import java.util.Arrays;

/**
 * Main class with iterations of Matching Pursuit Alghorithm.
 * 
 * @author Tom ondk
 */
public class MPFFT 
{	
	/**
	 * All available bases.
	 */
	private Base[] bases;
	
	/**
	 * Aghorithm instance constructor.
	 * 
	 * @param bases all available bases
	 */
	public MPFFT(Base ... bases)
	{
		this.bases = bases;
	}
	
	/**
	 * Does Matching Pursuit. Aproximates signal using input dictionary of bases.
	 * Fast Fourier Transformation is used to speeding up, therefore input signal length must be
	 * power of two.
	 *  
	 * @param signal input signal of length power of 2
	 * @param iterations number of iterations for alghorithm
	 * @return atoms used in approximation of signal
	 * @throws IllegalArgumentException on wrong signal length (not power of 2) or illegal iteration count.
	 */
	public DecompositionCollection doMP(double[] signal, int iterations) throws IllegalArgumentException
	{
		//BEGIN - check input params
		/*
		 * lenght of signal power of two
		 * number of iterations positive number
		 * 
		 */
		double nDouble = (Math.log(signal.length)) / Math.log(2);
		long nLong = Math.round((Math.log(signal.length)) / Math.log(2));
		final double ALLOWANCE = 1e-32;
		
		if ((Math.abs(nDouble - nLong)) > ALLOWANCE) {
			throw new IllegalArgumentException("Length of input signal must be power of 2.");
		}
		
		if (iterations <= 0) {
			throw new IllegalArgumentException("Number of iterations must be higher than 0.");
		}
		//END - check params
		
		// initialize variables
		DecompositionCollection collection = new DecompositionCollection(Arrays.copyOf(signal, signal.length));
		
		Atom currentAtom = null;
		Atom bestAtom;
		
		// find all atoms
		for (int i = 0; i < iterations; i++)
		{
			bestAtom = null;
			
			// multiple bases may be used - get the best
			for (Base b: bases)
			{
				currentAtom = b.getOptimalAtom(signal);
				
				if (bestAtom == null || bestAtom.getDifference() > currentAtom.getDifference())
				{
					bestAtom = currentAtom;
				}
			}
			
			collection.addAtom(bestAtom);
			bestAtom.subtractFrom(signal);
		}
		
		// ewruen ewa
		return collection;
	}
}
