package cz.zcu.kiv.eegdsp.wavelet.common;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/**
 * Generator of wavelets for wavelet transformation.
 * @author Petr Kellnhofer, A10N0018P
 * @date 2.1.2011
 *
 */
public abstract class WaveletGenerator {
	
	/**
	 * Gets names of all available wavelets.
	 * @return names of all available wavelets
	 */
	public String[] getWaveletNames() {
		return this.getWaveletNames(this.getWaveletClasses());
	}
	
	/**
	 * Gets wavelet instance by its name.
	 * 
	 * @param name name of wavelet
	 * @return wavelet instance
	 */
	public abstract Object getWaveletByName(String name);
	
	/**
	 * Gets all available wavelet classes.
	 * @return all available wavelet classes
	 */
	protected abstract Class<?>[] getWaveletClasses();
	
	/**
	 * Gets name list of all wavelet specified by its classes.
	 * 
	 * @param classes classes of wavelets
	 * @return list of wavelet names
	 */
	protected String[] getWaveletNames(Class<?>[] classes) {
		String[] list = new String[classes.length];
		
		for (int i = 0; i < classes.length; i++) {
			list[i] = this.getClassName(classes[i]);
		}
		
		return list;
	}
	
	/**
	 * Gets wavelet instance by its name.
	 * 
	 * @param name name of wavelet
	 * @param classes classes of available wavelets
	 * @return instannce of wavelet
	 * @throws IllegalArgumentException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 */
	protected Object getWaveletByName(String name, Class<?>[] classes) throws IllegalArgumentException, InstantiationException, IllegalAccessException, InvocationTargetException {
		Class<?> winner = null;

		// find class by name
		for (Class<?> cl : classes) {
			if (name.equals(this.getClassName(cl))) {
				winner = cl;
				break;
			}
		}
		
		// not found
		if (winner == null) {
			return null;
		}
		
		Constructor<?>[] constructors = winner.getConstructors();
		
		// find proper constructor (without parameters)
		for (Constructor<?> constructor : constructors) {
			Class<?>[] params = constructor.getParameterTypes();
			
			// not parameterless constructor
			if (params.length > 0) {
				continue;
			}
			
			// instancialize
			Object instance = constructor.newInstance(new Object[0]);
			return instance;
		}
		
		return null;
	}
	
	/**
	 * Gets name of class.
	 * @param cl class
	 * @return name of class
	 */
	protected String getClassName(Class<?> cl) {
		return cl.getSimpleName();
	}
}
