package cz.zcu.kiv.eegdsp.wavelet.continuous;


import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.math.complex.Complex;

import cz.zcu.kiv.eegdsp.wavelet.common.WaveletResult;
import cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.CWT;

/**
 * Result of continuous wavelet transformation.
 * 
 * @author Petr Kellnhofer, A10N0018P
 * @date 30.12.2010
 */
public class WaveletResultContinuous extends WaveletResult {
	
	/**
	 * Real coefficients from CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length 
	 */
	private double[][] real;	
	
	/**
	 * Gets real coefficients from CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 * 
	 * @return real coefficients array
	 */
	public double[][] getReal() {
		return real;
	}

	/**
	 * Imaginary coefficients from CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 */
	private double[][] imaginary;
	
	/**
	 * Gets imaginary coefficients from CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 * 
	 * @return coefficients from CWT
	 */
	public double[][] getImaginary() {
		return imaginary;
	}

	/**
	 * Array of modulus (size) coefficients from complex CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 */
	private double[][] modulus;
	
	/**
	 * Gets array of modulus (size) coefficients from complex CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 * 
	 * @return array of modulus (size) coefficients from complex CWT
	 */
	public double[][] getModulus() {
		return modulus;
	}
	
	/**
	 * Array of angle coefficients from complex CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 */
	private double[][] angles;

	/**
	 * Gets array of angle coefficients from complex CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 * 
	 * @return array of angle coefficients from complex CWT
	 */
	public double[][] getAngles() {
		return angles;
	}

	/**
	 * Array of highest coefficients.
	 * 
	 * Highest absolute value of real part in real data.
	 * One value for each step on scale.
	 * 
	 * First dimension: steps on scale
	 */
	private double[] highestCoefficients;
	
	/**
	 * Gets array of highest coefficients.
	 * 
	 * Highest absolute value of real part in real data.
	 * One value for each step on scale.
	 * 
	 * First dimension: steps on scale
	 * 
	 * @return array of highest coefficients
	 */
	public double[] getHighestCoefficients() {
		return highestCoefficients;
	}

	/**
	 * Array of indices of highest coefficients.
	 * 
	 * Index to corresponding second dimension of real data 
	 * array with highest real absolute value.
	 * 
	 * First dimension: steps on scale
	 */
	private int[] indicesHighestCoefficients;
	
	/**
	 * Gets array of indices of highest coefficients.
	 * 
	 * Index to corresponding second dimension of real data 
	 * array with highest real absolute value.
	 * 
	 * First dimension: steps on scale
	 * 
	 * @return array of highest indices
	 */
	public int[] getIndicesHighestCoefficients() {
		return indicesHighestCoefficients;
	}

	/**
	 * Creates result structure from CWT.
	 * 
	 * @param cwt
	 */
	public WaveletResultContinuous(CWT cwt) {
		this.real = cwt.getCwtDataReal();
		this.imaginary = cwt.getCwtDataImag();
		this.modulus = cwt.getModulusCwtData();
		this.angles = cwt.getAngleCwtData();
		
		this.highestCoefficients = cwt.getHighestCoeficients();
		this.indicesHighestCoefficients = cwt.getIndexesHighestCoeficients();
	}
	
	/**
	 * Gets array of complex coefficients from complex CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 * 
	 * @return array of complex coefficients from complex CWT
	 */
	public Complex[][] getComplex() {
		if (this.real == null || this.imaginary == null) {
			return null;
		}
		
		Complex[][] values = new Complex[this.real.length][this.real[0].length];
		
		for (int i = 0; i < this.real.length; i ++) {
			for (int j = 0; j < this.real[0].length; j ++) {
				values[i][j] = new Complex(this.real[i][j], this.imaginary[i][j]);
			}
		}
		
		return values;
	}

	/**
	 * Extracts data to array of two dimensional arrays
	 * 
	 * 0 => real
	 * 1 => imag
	 * 2 => modulus
	 * 3 => angle
	 * 4 => max coef
	 * 5 => indices of max coef
	 * 
	 * @return array of two dimensional arrays
	 */
	@Override
	public double[][][] toArray() {
		double[][][] res = new double[6][][];
		
		if (this.real != null) {
			res[0] = this.real.clone();
		}
		
		if (this.imaginary != null) {
			res[1] = this.imaginary.clone();
		}
		
		if (this.modulus != null) {
			res[2] = this.modulus.clone();
		}
		
		if (this.angles != null) {
			res[3] = this.angles.clone();
		}
		
		if (this.highestCoefficients != null) {
			res[4] = new double[1][];
			res[4][0] = this.highestCoefficients.clone();
		}
		
		if (this.indicesHighestCoefficients != null) {
			res[5] = new double[1][this.indicesHighestCoefficients.length];
			for (int i = 0; i < this.indicesHighestCoefficients.length; i++) {
				res[5][0][i] = (double)this.indicesHighestCoefficients[i];
			}
		}
		
		return res;
	}

	/**
	 * Extracts data to map of two dimensional arrays.
	 * 
	 * @return map of two dimensional arrays
	 */
	@Override
	public Map<String, Double[][]> toHashMap() {
		HashMap<String, Double[][]> map = new HashMap<String, Double[][]>();
				
		if (this.real != null) {
			map.put("real", this.packArray(this.real));
		}
		
		if (this.imaginary != null) {
			map.put("imaginary", this.packArray(this.imaginary));
		}
		
		if (this.modulus != null) {
			map.put("modulus", this.packArray(this.modulus));
		}
		
		if (this.angles != null) {
			map.put("angles", this.packArray(this.angles));
		}
		
		if (this.highestCoefficients != null) {
			Double[][] temp = new Double[1][];
			temp[0] = this.packArray(this.highestCoefficients);
			map.put("highestCoeficients", temp);
		}
		
		if (this.indicesHighestCoefficients != null) {
			Double[][] temp = new Double[1][this.indicesHighestCoefficients.length];
			for (int i = 0; i < this.indicesHighestCoefficients.length; i++) {
				temp[0][i] = Double.valueOf(this.indicesHighestCoefficients[i]);
			}
			
			map.put("indexesHighestCoeficients", temp);
		}
		
		return map;
	}

	/**
	 * Prints data do string.
	 * 
	 * @return string
	 */
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		
		builder.append("WaveletResultContinuous:\n");
		
		if (this.real != null) {
			builder.append("\tReal: " + this.arrayToString(this.real) + "\n");
		}
		
		if (this.imaginary != null) {
			builder.append("\tImaginary: " + this.arrayToString(this.imaginary) + "\n");
		}
		
		if (this.modulus != null) {
			builder.append("\tModulus: " + this.arrayToString(this.modulus) + "\n");
		}
		
		if (this.angles != null) {
			builder.append("\tAngles: " + this.arrayToString(this.angles) + "\n");
		}
		
		if (this.highestCoefficients != null) {
			builder.append("\tHighest coefficients: " + Arrays.toString(this.highestCoefficients) + "\n");
		}
		
		if (this.indicesHighestCoefficients != null) {
			builder.append("\tIndices highest coefficients: " + Arrays.toString(this.indicesHighestCoefficients) + "\n");
		}
		
		return builder.toString();
	}
	
	private String arrayToString(double[][] array) {
		StringBuilder builder = new StringBuilder();
		builder.append("{\n");
		
		for (double[] subarray : array) {
			builder.append(Arrays.toString(subarray));
			builder.append("\n");
		}
		
		builder.append("}");
		return builder.toString();
	}
}
