package cz.zcu.kiv.eegdsp.wavelet.continuous;

import cz.zcu.kiv.eegdsp.wavelet.common.WaveletTransformation;
import cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.CWT;
import cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.wavelets.ComplexGaussian;
import cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.wavelets.WaveletCWT;

/**
 * Execution interface for continuous wavelet transformation.
 * Singleton.
 * 
 * @author Petr Kellnhofer, A10N0018P
 * @date 2.1.2011
 *
 */
public class WaveletTransformationContinuous extends WaveletTransformation {
	
	/**
	 * Signleton instance.
	 */
	private static WaveletTransformationContinuous instance;
	
	/**
	 * Wavelet used.
	 */
	private WaveletCWT wavelet;
	
	/**
	 * Gets wavelet used for CWT.
	 * 
	 * @return wavelet
	 */
	public WaveletCWT getWavelet() {
		return wavelet;
	}

	/**
	 * Sets wavelt used for CWT.
	 * @param wavelet
	 */
	public void setWavelet(WaveletCWT wavelet) {
		if (wavelet == null) {
			throw new IllegalArgumentException("Wavelet must be provided.");
		}
		
		this.wavelet = wavelet;
	}
	
	/**
	 * Bottom limit of scale.
	 */
	private double minScale;
	
	/**
	 * Gets bottom limit of scale.
	 * 
	 * @return bottom limit of scale
	 */
	public double getMinScale() {
		return minScale;
	}

	/**
	 * Sets bottom limit of scale.
	 * 
	 * @param minScale bottom limit of scale
	 */
	public void setMinScale(double minScale) {
		if (minScale >= this.maxScale) {
			throw new IllegalArgumentException("Scale min must be lower than step max.");
		}
		
		this.minScale = minScale;
	}

	/**
	 * Upper limit of scale.
	 */
	private double maxScale;

	/**
	 * Gets upper limit of scale.
	 *  
	 * @return upper limit of scale
	 */
	public double getMaxScale() {
		return maxScale;
	}

	/**
	 * Sets upper limit of scale.
	 * 
	 * @param maxScale upper limit of scale
	 */
	public void setMaxScale(double maxScale) {
		if (maxScale <= this.minScale) {
			throw new IllegalArgumentException("Scale max must be higher than step min.");
		}
		
		this.maxScale = maxScale;
	}

	/**
	 * Step length on scale.
	 */
	private double stepScale;

	/**
	 * Gets step length on scale.
	 * 
	 * @return step length on scale
	 */
	public double getStepScale() {
		return stepScale;
	}

	/**
	 * Sets step length on scale.
	 * 
	 * @param stepScale step length on scale
	 */
	public void setStepScale(double stepScale) {
		if (stepScale <= 0) {
			throw new IllegalArgumentException("Step length must be positive real number.");
		}
		
		this.stepScale = stepScale;
	}

	/*
	 * Constructors. 
	 */
	
	/**
	 * Private constructor of this singleton.
	 * Uses ComplexGaussian.
	 */
	private WaveletTransformationContinuous() {
		this.minScale = 0.1;
		this.maxScale = 10;
		this.stepScale = 0.1;
		
		this.setWavelet(new ComplexGaussian());
	}
	
	/**
	 * Processes signal.
	 * 
	 * @param signal input signal
	 * @return result
	 */
	@Override
	public WaveletResultContinuous processSignal(double[] signal) {
		if (signal == null) {
			throw new IllegalArgumentException("Signal must be provided.");
		}
		
		try {
			CWT cwt = new CWT(this.minScale, this.maxScale, this.stepScale, this.wavelet);
			cwt.transform(signal);
			
			WaveletResultContinuous result = new WaveletResultContinuous(cwt);
			return result;
			
		} catch (Exception e) {
			String message = "Signal processing failed.";
			message += "\nInner message: " + e.getMessage();
			
			throw new ArithmeticException(message);
		}		
	}
	
	/**
	 * Gets generator of continuous wavelets.
	 * 
	 * @return generator
	 */
	@Override
	public WaveletGeneratorContinuous getWaveletGenerator() {
		return WaveletGeneratorContinuous.getInstance();
	}
	
	/**
	 * Gets singleton instance.
	 * 
	 * @return signleton instance
	 */
	public static WaveletTransformationContinuous getInstance() {
		if (instance == null) {
			instance = new WaveletTransformationContinuous();
		}
		
		return instance;
	}
}
