package cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm;

import cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.wavelets.WaveletCWT;

/**
 * Continuous wavelet transformation class.
 */
public class CWT
{
	/*
	 * Transformation and signal parameters.
	 */
	
	/**
	 * Bottom limit of scale.
	 */
	private double minScale;
	
	/**
	 * Gets bottom limit of scale.
	 * 
	 * @return bottom limit of scale
	 */
	public double getMinScale()
	{
		return minScale;
	}
	
	/**
	 * Sets bottom limit of scale.
	 * 
	 * @param minScale bottom limit of scale
	 */
	public void setMinScale(double minScale)
	{
		if (minScale >= this.maxScale) {
			throw new IllegalArgumentException("Scale min must be lower than step max.");
		}
		
		this.minScale = minScale;
	}
	
	/**
	 * Upper limit of scale.
	 */
	private double maxScale;
	
	/**
	 * Gets upper limit of scale.
	 *  
	 * @return upper limit of scale
	 */
	public double getMaxScale()
	{
		return maxScale;
	}
	
	/**
	 * Sets upper limit of scale.
	 * 
	 * @param maxScale upper limit of scale
	 */
	public void setMaxScale(double maxScale)
	{
		if (maxScale <= this.minScale) {
			throw new IllegalArgumentException("Scale max must be higher than step min.");
		}
		
		this.maxScale = maxScale;
	}
	
	/**
	 * Step length on scale.
	 */
	private double stepScale;
	
	/**
	 * Gets step length on scale.
	 * 
	 * @return step length on scale
	 */
	public double getStepScale()
	{
		return stepScale;
	}
	
	/**
	 * Sets step length on scale.
	 * 
	 * @param stepScale step length on scale
	 */
	public void setStepScale(double stepScale)
	{
		if (stepScale <= 0) {
			throw new IllegalArgumentException("Step length must be positive real number.");
		}
		
		this.stepScale = stepScale;
	}
	
	/**
	 * Mother wavelet used.
	 */
	private WaveletCWT wavelet;
	
	/**
	 * Gets mother wavelet used.
	 * 
	 * @return mother wavelet
	 */
	public WaveletCWT getWavelet()
	{
		return wavelet;
	}
	
	/**
	 * Sets mother wavelet used.
	 * 
	 * @param wavelet mother wavelet
	 */
	public void setWavelet(WaveletCWT wavelet)
	{
		if (wavelet == null) {
			throw new IllegalArgumentException();
		}
		
		this.wavelet = wavelet;
	}
	
	
	/*
	 * Transformation results.
	 */
	
	/**
	 * Real coefficients from CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length 
	 */
	private double[][] cwtDataReal;	
	
	/**
	 * Gets real coefficients from CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 * 
	 * @return real coefficients array
	 */
	public double[][] getCwtDataReal()
	{
		return cwtDataReal;
	}
	
	/**
	 * Imaginary coefficients from CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 */
	private double[][] cwtDataImag;
	
	/**
	 * Gets imaginary coefficients from CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 * 
	 * @return coefficients from CWT
	 */
	public double[][] getCwtDataImag()
	{		
		return cwtDataImag;
	}
	
	/**
	 * Array of modulus (size) coefficients from complex CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 */
	private double[][] modulusCwtData;	
	
	/**
	 * Gets array of modulus (size) coefficients from complex CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 * 
	 * @return array of modulus (size) coefficients from complex CWT
	 */
	public double[][] getModulusCwtData()
	{		
		return modulusCwtData;
	}
	
	/**
	 * Array of angle coefficients from complex CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 */
	private double[][] angleCwtData;
	
	/**
	 * Gets array of angle coefficients from complex CWT.
	 * 
	 * First dimension: steps on scale
	 * Second dimension: signal length
	 * 
	 * @return array of angle coefficients from complex CWT
	 */
	public double[][] getAngleCwtData()
	{
		return angleCwtData;
	}
	
	/**
	 * Array of highest coefficients.
	 * 
	 * Highest absolute value of real part in real data.
	 * One value for each step on scale.
	 * 
	 * First dimension: steps on scale
	 */
	private double[] highestCoefficients;	
	
	/**
	 * Gets array of highest coefficients.
	 * 
	 * Highest absolute value of real part in real data.
	 * One value for each step on scale.
	 * 
	 * First dimension: steps on scale
	 * 
	 * @return array of highest coefficients
	 */
	public double[] getHighestCoeficients()
	{
		return highestCoefficients;
	}
	
	/**
	 * Array of indices of highest coefficients.
	 * 
	 * Index to corresponding second dimension of real data 
	 * array with highest real absolute value.
	 * 
	 * First dimension: steps on scale
	 */
	private int[] indicesHighestCoefficients;
	
	/**
	 * Gets array of indices of highest coefficients.
	 * 
	 * Index to corresponding second dimension of real data 
	 * array with highest real absolute value.
	 * 
	 * First dimension: steps on scale
	 * 
	 * @return array of highest indices
	 */
	public int[] getIndexesHighestCoeficients()
	{
		return indicesHighestCoefficients;
	}
	
	/*
	 * 
	 * Other settings.
	 * 
	 */
	
	/**
	 * Switch enabling transformation.
	 * Default value: true.
	 */
	private boolean transformEnabled;
	
	/**
	 * Gets setting of transformation enabled.
	 * 
	 * @return setting of transformation enabled
	 */
	public void getTransformEnabled(boolean enabled)
	{
		transformEnabled = enabled;
	}
	
	/**
	 * Sets setting of transformation enabled.
	 * 
	 * @return setting of transformation enabled
	 */
	public void setTransformEnabled(boolean enabled)
	{
		transformEnabled = enabled;
	}
	
	/*
	 * 
	 * Constructors.
	 * 
	 */
	
	/**
	 * Constructor. Enables transformation.
	 * 
	 * @param minScale - bottom limit of scale
	 * @param maxScale - upper limit of scale
	 * @param stepScale - step of scale
	 * @param wavelet - mother wavelet
	 */
	public CWT(double minScale, double maxScale, double stepScale, WaveletCWT wavelet)
	{
		this.minScale = minScale;
		this.maxScale = maxScale;
		this.stepScale = stepScale;
		
		// set again to check conditions
		this.setMaxScale(maxScale);
		this.setMinScale(minScale);
		this.setStepScale(stepScale);
		this.setWavelet(wavelet);
		this.setTransformEnabled(true);
	}
	
	/*
	 * 
	 * Main code.
	 * 
	 */
	
	/**
	 * Continuous wavelet transformation
	 * 
	 * @param inputSignal - input signal
	 * @return array of coefficients
	 */
	public void transform(double[] inputSignal)
	{
		// initialize data structures
		cwtDataReal = new double[(int) ((maxScale - minScale)/stepScale) + 1][inputSignal.length];
		cwtDataImag = new double[cwtDataReal.length][inputSignal.length];
		modulusCwtData = new double[cwtDataReal.length][inputSignal.length];
		angleCwtData = new double[cwtDataReal.length][inputSignal.length];
		highestCoefficients = new double[cwtDataReal.length];
		indicesHighestCoefficients = new int[cwtDataReal.length];
		
		// local variables
		int b, x, scale = 0;
		double t, a;
		double highestCoef = 0; 
		int indexHighestCoef = 0;
		
		// go through scale
		for (a = minScale; a <= maxScale; a += stepScale)
		{
			// remember extremes
			highestCoef = 0; 
			indexHighestCoef = 0;

			// simulation mode
			if(!transformEnabled)
				return;
			
			// go through signal
			for (b = 0; b < inputSignal.length; b++)
			{
				cwtDataReal[scale][b] = 0;
				cwtDataImag[scale][b] = 0;

				// go through signal
				for (x = 0; x < inputSignal.length; x++)
				{						
					t = (x - b)/a;
					
					cwtDataReal[scale][b] += inputSignal[x] * wavelet.reCoef(t, a);
					cwtDataImag[scale][b] += inputSignal[x] * wavelet.imCoef(t, a);
				}		
				
				// mark extreme
				if(highestCoef < Math.abs(cwtDataReal[scale][b]))
				{
					highestCoef = Math.abs(cwtDataReal[scale][b]);
					indexHighestCoef = b;
				}
				
				// calculate angles 
				modulusCwtData[scale][b] = Math.sqrt(cwtDataReal[scale][b] * cwtDataReal[scale][b] + cwtDataImag[scale][b] * cwtDataImag[scale][b]);
				angleCwtData[scale][b] = Math.atan2(cwtDataImag[scale][b],cwtDataReal[scale][b]);
			}	
			
			highestCoefficients[scale] = highestCoef;
			indicesHighestCoefficients[scale] = indexHighestCoef;
			
			scale++;
		}
	}
		
	/**
	 * Sets all parameters of signal at once
	 * 
	 * @param minScale bottom scale limit
	 * @param maxScale upper scale limit
	 * @param stepScale step size
	 * @param wavelet mother wavelet used in transformation
	 */
	public void setCwtParameters(double minScale, double maxScale, double stepScale, WaveletCWT wavelet)
	{
		this.maxScale = maxScale;
		this.minScale = minScale;
		this.stepScale = stepScale;
		this.wavelet = wavelet;
	}
}
