package cz.zcu.kiv.eegdsp.wavelet.discrete;


import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import cz.zcu.kiv.eegdsp.wavelet.common.WaveletResult;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.DWT;

/**
 * Result of discrete wavelet transformation.
 * 
 * @author Petr Kellnhofer, A10N0018P
 * @date 2.1.2011
 *
 */
public class WaveletResultDiscrete extends WaveletResult {
	
	/**
	 * Discrete wavelet transformation coefficients.
	 */
	private double[] dwtCoefficients;
	
	/**
	 * Gets discrete wavelet transformation coefficients.
	 * 
	 * @return discrete wavelet transformation coefficients
	 */
	public double[] getDwtCoefficients() {
		return dwtCoefficients;
	}

	/**
	 * Reconstructed signal from inverse discrete wavelet transformation.
	 */
	private double[] reconstructedSignal;
	
	/**
	 * Gets reconstructed signal from inverse discrete wavelet transformation.
	 * 
	 * @return reconstructed signal from inverse discrete wavelet transformation
	 */
	public double[] getReconstructedSignal() {
		return reconstructedSignal;
	}
	
	/**
	 * Array of the highest abs coefficients
	 * for each level of decomposition.
	 * 
	 * Length equals level of decomposition.
	 */
	private double[] highestCoefficients;

	/**
	 * Gets an array of the highest abs coefficients
	 * for each level of decomposition.
	 * 
	 * Length equals level of decomposition.
	 * 
	 * @return array of the highest coefficients
	 */
	public double[] getHighestCoefficients() {
		return highestCoefficients;
	}
	
	/**
	 * Array of indices to DWT coefficient array
	 * where highest abs values for each level of decomposition can be found can be found.
	 * 
	 * Length equals level of decomposition.
	 */
	private int[] indicesHighestCoefficients;

	/**
	 * Gets an array of indices to DWT coefficient array
	 * where highest abs values for each level of decomposition can be found can be found.
	 * 
	 * Length equals level of decomposition.
	 * 
	 * @return array of indices to DWT coefficient array
	 */
	public int[] getIndicesHighestCoefficients() {
		return indicesHighestCoefficients;
	}
	
	/**
	 * Level of decomposition.
	 */
	private int levelOfDecomposition;
		
	/**
	 * Gets level of decomposition.
	 * 
	 * @return level of decomposition
	 */
	public int getLevelOfDecomposition() {
		return levelOfDecomposition;
	}
	
	/**
	 * Extract discrete wavelet transformation results from algorithm class.
	 * 
	 * @param dwt algorithm class for DWT
	 */
	public WaveletResultDiscrete(DWT dwt) {
		this.dwtCoefficients = dwt.getDwtCoefficients();
		this.reconstructedSignal = dwt.getReconstructedSignal();
		
		this.highestCoefficients = dwt.getHighestCoefficients();
		this.indicesHighestCoefficients = dwt.getIndicesHighestCoefficients();
		
		this.levelOfDecomposition = dwt.getLevelsOfDecompositon();
	}

	/**
	 * Extracts data to array of two dimensional arrays
	 * 
	 * 0 => dwtCoefficients
	 * 1 => reconstructedSignal
	 * 2 => highestCoefficients
	 * 3 => indicesHighestCoefficients
	 * 4 => levelOfDecomposition
	 * 
	 * @return array of two dimensional arrays
	 */
	@Override
	public double[][][] toArray() {
		double[][][] res = new double[5][1][];
		
		if (this.dwtCoefficients != null) {
			res[0][0] = this.dwtCoefficients.clone();
		}
		
		if (this.reconstructedSignal != null) {
			res[1][0] = this.reconstructedSignal.clone();
		}
		
		if (this.highestCoefficients != null) {
			res[2][0] = this.highestCoefficients.clone();
		}
		
		if (this.indicesHighestCoefficients != null) {
			res[3][0] = new double[this.indicesHighestCoefficients.length];
			for (int i = 0; i < this.indicesHighestCoefficients.length; i++) {
				res[3][0][i] = this.indicesHighestCoefficients[i];
			}
		}
		
		res[4][0] = new double[1];
		res[4][0][0] = this.levelOfDecomposition;
		
		return res;
	}

	/**
	 * Extracts data to map of two dimensional arrays.
	 * 
	 * @return map of two dimensional arrays
	 */
	@Override
	public Map<String, Double[][]> toHashMap() {
		HashMap<String, Double[][]> map = new HashMap<String, Double[][]>();
				
		if (this.dwtCoefficients != null) {
			map.put("dwtCoefficients", new Double[][] { this.packArray(this.dwtCoefficients) });
		}
		
		if (this.reconstructedSignal != null) {
			map.put("reconstructedSignal", new Double[][] { this.packArray(this.reconstructedSignal) });
		}
		
		if (this.highestCoefficients != null) {
			map.put("highestCoefficients", new Double[][] { this.packArray(this.highestCoefficients) });
		}
		
		if (this.indicesHighestCoefficients != null) {
			map.put("indicesHighestCoefficients", new Double[][] { this.packArray(this.indicesHighestCoefficients) });
		}
		
		map.put("levelOfDecomposition", new Double[][] { new Double[] { Double.valueOf(this.levelOfDecomposition) } });
		
		return map;
	}

	/**
	 * Prints data do string.
	 * 
	 * @return string
	 */
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		
		builder.append("WaveletResultDiscrete:\n");
		
		if (this.dwtCoefficients != null) {
			builder.append("\tDWT coefficients: " + Arrays.toString(this.dwtCoefficients) + "\n");
		}
		
		if (this.reconstructedSignal != null) {
			builder.append("\tReconstructed signal: " + Arrays.toString(this.reconstructedSignal) + "\n");
		}
		
		if (this.highestCoefficients != null) {
			builder.append("\tHighest coefficients: " + Arrays.toString(this.highestCoefficients) + "\n");
		}
		
		if (this.indicesHighestCoefficients != null) {
			builder.append("\tHighest coefficient indices: " + Arrays.toString(this.indicesHighestCoefficients) + "\n");
		}
		
		builder.append("\tHighest coefficients: " + this.levelOfDecomposition + "\n");
				
		return builder.toString();
	}
}
