package cz.zcu.kiv.eegdsp.wavelet.discrete;

import cz.zcu.kiv.eegdsp.wavelet.common.WaveletTransformation;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.DWT;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Symmlet4;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.WaveletDWT;

/**
 * Execution interface for discrete wavelet transformation.
 * Singleton.
 * 
 * @author Petr Kellnhofer, A10N0018P
 * @date 2.1.2011
 *
 */
public class WaveletTransformationDiscrete extends WaveletTransformation {
	
	/**
	 * Private singleton instace.
	 */
	private static WaveletTransformationDiscrete instance;
	
	/**
	 * Wavelet used for transformation.
	 */
	private WaveletDWT wavelet;
	
	/**
	 * Gets wavelet used for transformation.
	 * 
	 * @return wavelet used for transformation
	 */
	public WaveletDWT getWavelet() {
		return wavelet;
	}
	
	/**
	 * Sets wavelet used for transformation.
	 * 
	 * @param wavelet wavelet used for transformation
	 */
	public void setWavelet(WaveletDWT wavelet) {
		if (wavelet == null) {
			throw new IllegalArgumentException("Wavelet must be provided.");
		}
		
		this.wavelet = wavelet;
	}
	
	/**
	 * Private constructor using Symmlet4 wavelet.
	 */
	private WaveletTransformationDiscrete() {
		this.setWavelet(new Symmlet4());
	}
	
	/**
	 * Processes signal using discrete wavelet transformation.
	 * 
	 * @param signal input signal
	 * @return result
	 */
	@Override
	public WaveletResultDiscrete processSignal(double[] signal) {
		if (signal == null) {
			throw new IllegalArgumentException("Signal must be provided.");
		}
		
		try {
			DWT dwt = new DWT(this.wavelet);
			dwt.transform(signal);
			
			WaveletResultDiscrete result = new WaveletResultDiscrete(dwt);
			return result;
			
		} catch (Exception e) {
			String message = "Signal processing failed.";
			message += "\nInner message: " + e.getMessage();
			
			throw new ArithmeticException(message);
		}	
	}
	
	/**
	 * Reconstructs original signal from coefficients created discrete wavelet decomposition.
	 * Inverse DWT.
	 * 
	 * @param coefficients coefficients from DWT
	 * @return reconstructed signal
	 */
	public WaveletResultDiscrete reconstructSignal(double[] coefficients) {
		if (coefficients == null) {
			throw new IllegalArgumentException("Signal must be provided.");
		}
		
		try {
			DWT dwt = new DWT(this.wavelet);
			dwt.invTransform(coefficients);
			
			WaveletResultDiscrete result = new WaveletResultDiscrete(dwt);
			return result;
			
		} catch (Exception e) {
			String message = "Signal processing failed.";
			message += "\nInner message: " + e.getMessage();
			
			throw new ArithmeticException(message);
		}	
	}
	
	/**
	 * Gets generator of discrete wavelets.
	 * 
	 * @return generator of discrete wavelets
	 */
	@Override
	public WaveletGeneratorDiscrete getWaveletGenerator() {
		return WaveletGeneratorDiscrete.getInstance();
	}
	
	/**
	 * Gets singleton instance.
	 * @return singleton instance
	 */
	public static WaveletTransformationDiscrete getInstance() {
		if (instance == null) {
			instance = new WaveletTransformationDiscrete();
		}
		
		return instance;
	}
}
