package cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm;


import java.util.Arrays;

import cz.zcu.kiv.eegdsp.utils.Mathematic;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.WaveletDWT;

/**
 * Class of discrete wavelet transformation.
 */
public class DWT
{
	/*
	 * Inputs and settings 
	 */
	
	/**
	 * Wavelet used for transformation.
	 */
	private WaveletDWT wavelet;
	
	/**
	 * Gets wavelet used for transformation.
	 * 
	 * @return wavelet used for transformation
	 */
	public WaveletDWT getWavelet()
	{
		return wavelet;
	}

	/**
	 * Sets wavelet used for transformation.
	 * 
	 * @param wavelet wavelet used for transformation
	 */
	public void setWavelet(WaveletDWT wavelet) 
	{
		if (wavelet == null) {
			throw new IllegalArgumentException();
		}
		
		this.wavelet = wavelet;
	}
	
	/**
	 * Nearest bigger power of two searched from wavelet length up.
	 */
	private int majorWaveletLengthOfPow2;
	
	/*
	 * Results
	 */
	
	/**
	 * Discrete wavelet transformation coefficients.
	 */
	private double[] dwtCoefficients;
	
	/**
	 * Gets discrete wavelet transformation coefficients.
	 * 
	 * @return discrete wavelet transformation coefficients
	 */
	public double[] getDwtCoefficients() {
		return dwtCoefficients;
	}

	/**
	 * Reconstructed signal from inverse discrete wavelet transformation.
	 */
	private double[] reconstructedSignal;
	
	/**
	 * Gets reconstructed signal from inverse discrete wavelet transformation.
	 * 
	 * @return reconstructed signal from inverse discrete wavelet transformation
	 */
	public double[] getReconstructedSignal() {
		return reconstructedSignal;
	}

	/**
	 * Array of the highest abs coefficients
	 * for each level of decomposition.
	 * 
	 * Length equals level of decomposition.
	 */
	private double[] highestCoefficients;
	
	/**
	 * Gets array of the highest abs coefficients
	 * for each level of decomposition.
	 * 
	 * Length equals level of decomposition.
	 * 
	 * @return array of the highest coefficients
	 */
	public double[] getHighestCoefficients()
	{
		return highestCoefficients;
	}
	
	/**
	 * Array of indices to DWT coefficient array
	 * where highest abs values for each level of decomposition can be found can be found.
	 * 
	 * Length equals level of decomposition.
	 */
	private int[] indicesHighestCoefficients;		
	
	/**
	 * Gets an array of indices to DWT coefficient array
	 * where highest abs values for each level of decomposition can be found can be found.
	 * 
	 * Length equals level of decomposition.
	 * 
	 * @return array of indices to DWT coefficient array
	 */
	public int[] getIndicesHighestCoefficients()
	{
		return indicesHighestCoefficients;
	}
	
	/**
	 * Constructor of DWT alghorithm class.
	 * 
	 * @param wavelet wavelet used for transformation
	 */
	public DWT(WaveletDWT wavelet)
	{
		this.setWavelet(wavelet);
		this.majorWaveletLengthOfPow2 = this.wavelet.getMajorWaveletLengthOfPow2();
	}
	
	/**
	 * Processes signal, creates discrete wavelet transformation. Signal length must be equal
	 * or higher than length of wavelet.
	 * 
	 * Final transformation level is based on signal length.
	 * 
	 * @param inputSignal input signal
	 * @return DWT coefficients
	 */
	public double[] transform(double[] inputSignal)
	{		
		dwtCoefficients = signalPowerBase2(inputSignal);		
		int levelOfDecomposition = getLevelsOfDecompositon() - 1;
		highestCoefficients = new double[levelOfDecomposition + 1];
		indicesHighestCoefficients = new int[levelOfDecomposition + 1];
		
		// divide from full powered signal length down to wavelet length
		for (int last = dwtCoefficients.length; last >= majorWaveletLengthOfPow2; last /= 2)
		{
			transform(dwtCoefficients, last, levelOfDecomposition);
			levelOfDecomposition--;
		}
		
		return dwtCoefficients;
	}
	

	/**
	 * Gets level of decomposition after DWT.
	 * 
	 * @return level of decomposition of last DWT or -1 if none done yet
	 */
	public int getLevelsOfDecompositon()
	{
		if (this.dwtCoefficients == null) {
			return -1;
		}
		int powerOfData = (int)Mathematic.log2(dwtCoefficients.length);
		int powerOfWavelet = (int)Mathematic.log2(
				Mathematic.newMajorNumberOfPowerBase2(wavelet.getWaveletArray().length));
		
		if(powerOfData - (powerOfWavelet - 1) < 0) {
			return 0;
		}
		else {
			return powerOfData - (powerOfWavelet - 1);
		}
	}

	/**
	 * Inverse DWT. DWT coefficients must have equal or bigger length than wavelet used.  
	 * 
	 * @param inputCoef DWT coefficients
	 * @return reconstructed signal
	 */
	public double[] invTransform(double[] inputCoef)
	{
		reconstructedSignal = signalPowerBase2(inputCoef);
		
		for (int last = majorWaveletLengthOfPow2; last <= reconstructedSignal.length; last *= 2)
		{
			invTransform(reconstructedSignal, last);
		}
		
		return reconstructedSignal;
	}
	
	/**
	 * Uses zero padding to resize length of input singal to nearest bigger power of 2.
	 * 
	 * @param inputSignal input signal
	 */ 
	private double[] signalPowerBase2(double[] inputSignal)
	{
		int newSignalLength = Mathematic.newMajorNumberOfPowerBase2(inputSignal.length);
		
		double[] signal = new double[newSignalLength];
		
		if(newSignalLength != inputSignal.length)
		{						
			signal = Arrays.copyOf(inputSignal, newSignalLength);
			Arrays.fill(signal, inputSignal.length, signal.length, Mathematic.ZERO);
		}
		else
			signal = inputSignal.clone();
		
		return signal;
	}
	
	/**
	 * Transforms signal using wavelet.
	 * 
	 * First half of transformed signal used for approximated part.
	 * Second half of transformed signal used for detail part from wavelet coefficients.	 * 
	 * 
	 * @param signal input signal
	 * @param last divider of two signal parts (processed up to last)
	 * @param levelOfDecomposition level of decomposition
	 */
	private void transform(double[] signal, int last, int levelOfDecomposition)
	{
		int half = last/2;
		double tmp[] = new double[last];
		int i = 0, j, k;
		double highestCoef = 0;
		int indexHighesCoef = 0;
		
		double[] scaleCoefs = this.wavelet.getScaleArray();
		double[] waveletCoefs = this.wavelet.getWaveletArray();
			
		// for every second value from first half
		for (j = 0; j < last; j += 2)
		{
			// through wavelet coefficients
			for(k = 0; k < scaleCoefs.length; k++)
			{
				tmp[i] += signal[(j+k)%last] * scaleCoefs[k];					
				tmp[i + half] += signal[(j+k)%last] * waveletCoefs[k];
			}
			
			// mark highest coefficients
			if(highestCoef < Math.abs(tmp[i + half]))
			{
				highestCoef = Math.abs(tmp[i + half]);
				indexHighesCoef = i;
			}	
			
			i++;
		}
		
		// mark highest coefficients
		highestCoefficients[levelOfDecomposition] = highestCoef;
		indicesHighestCoefficients[levelOfDecomposition] = indexHighesCoef;
			
		for (i = 0; i < last; i++)
		{
			signal[i] = tmp[i];		
		}		
	} 

	/**
	 * Transforms coefficients back to original signal
	 * using wavelet.
	 * 
	 * Composes original signal from aproximation and detail part
	 * of transformed signal.
	 * 
	 * Example:
	 * tmp[0] = [s6....s0 s2 s4 | s7....s1 s3 s5]
	 * tmp[1] = [w6....w0 w2 w4 | w7....w1 w3 w5]
	 * ...
	 * tmp[.] = [s0 s2 s4 s6 .. | s1 s3 s5 s7 ..]
	 * tmp[.] = [w0 w2 w4 w6 .. | w1 w3 w5 w7 ..]
	 * ...
	 * tmp[n-2] = [.. s0 s2 s4 s6 | .. s1 s3 s5 s7]
	 * tmp[n-1] = [.. w0 w2 w4 w6 | .. w1 w3 w5 w7]
	 * 
	 * @param coef transformed signal (DWT coefficient)
	 * @param last length of transformed part of signal
	 */
	private void invTransform(double[] coef, int last)
	{
		double tmp[] = new double[last];
		int half = last/2;
		int i, j = 0, k, index;
		
		double[] scaleCoefs = this.wavelet.getIScaleArray();
		double[] waveletCoefs = this.wavelet.getIWaveletArray();
		
		i = half - ((this.wavelet.getLength()/2) - 1);
			
		// up to processed part
		for (index = 0; index < half; index++)
		{
			// through wavelet
			for(k = 0; k < scaleCoefs.length - 1; k += 2)
			{
				tmp[j] += 	coef[(i+(k/2))%half] * scaleCoefs[k] + 
							coef[((i+(k/2))%half) + half] * scaleCoefs[k + 1];
				
				tmp[j + 1] += 	coef[(i+(k/2))%half] * waveletCoefs[k] + 
								coef[((i+(k/2))%half) + half] * waveletCoefs[k + 1];
			}
			
			j += 2;
			
			i++;
		}
			
		for (i = 0; i < last; i++)
		{
			coef[i] = tmp[i];
		}		
	}
	
	
	
}